#ifdef PLATFORM_WINDOWS
	#define SL(s) Obelisk::CString(sizeof(s) - 1, L##s)
#else
	#define SL(s) Obelisk::CString(sizeof(s) - 1, s)
#endif

namespace Obelisk
{
	class CString
	{
	public:
		CString();
		CString(int aSize, const TCodeUnit* aData);
		CString(CString aString, int aCount);
		CString(const CString& aString);
		~CString();

		CString& operator =(const CString& aString);
		CString operator +(const CString& aString) const;
		CString& operator +=(const CString& aString);

		bool IsEmpty() { return mSize == 0; }
		bool IsCharacter();

		bool operator ==(const CString& aString) const;
		bool operator !=(const CString& aString) const;

		int Iterator(int) const;
		bool HasNext(int i) const { return i < mSize; }
		void Next(int&) const;
		int Last() const;
		bool HasPrev(int i) const { return i >= 0; }
		void Prev(int&) const;
		CString GetChar(int) const;

		const TCodeUnit* GetData() { return mData == NULL ? gEmptyString : mData; }
		int GetSize() { return mSize; }

	private:
		int *mReferenceCount;
		TCodeUnit *mData;
		int mSize;

		void Release()
		{
			if (mReferenceCount != NULL)
			{
				if (*mReferenceCount == 1)
				{
					delete mReferenceCount;
					delete [] mData;
				}
				else
					--*mReferenceCount;
			}
		}

		static int GetCodePointSize(const TCodeUnit* aData, int aSize, int aOffset);
		static int GetPrevCodePointSize(const TCodeUnit* aData, int aSize, int aOffset);
	};
}