namespace Obelisk
{
	class IThread;
	class IReader;
	class IWriter;
	class CTexture;
	class CVector;
	class CColour;

	enum EKey
	{
		kLeft, kRight, kUp, kDown, kEnter, kSpace, kEscape, kBackspace, kTab, kLeftShift,
		kRightShift, kLeftControl, kRightControl, kLeftAlt, kRightAlt, kA, kB, kC, kD, kE,
		kF, kG, kH, kI, kJ, kK, kL, kM, kN, kO, kP, kQ, kR, kS, kT, kU, kV, kW, kX, kY, kZ,
		k0, k1, k2, k3, k4, k5, k6, k7, k8, k9
	};

	class IObelisk
	{
	public:
		virtual void SetDirectory(Obelisk::CString aDirectory) = 0;
		virtual void SetCaption(CString aCaption) = 0;
		virtual void SetSize(int aXSize, int aYSize) = 0;

		virtual CString IntToString(int aNumber, int aLeading0s=0) = 0;

		virtual void RunThread(IThread* aThread) = 0;
		virtual void NotifyWindow(int aMessage, int aData) = 0;

		virtual void StartTimer() = 0;
		virtual void StopTimer() = 0;

		virtual unsigned int CreateTimer() = 0;
		virtual unsigned int Duration(unsigned int aTimer) = 0;

		virtual bool MouseButtonDown(bool aRight) = 0;
		virtual bool MouseButtonClicked(bool aRight) = 0;
		virtual bool MouseButtonReleased(bool aRight) = 0;
		virtual void GetCursorPos(int& aCursorX, int& aCursorY) = 0;

		virtual bool IsKeyDown(EKey aKey) = 0;

		virtual double Random() = 0;
		virtual int Random(int aMin, int aMax) = 0;

		// graphics functions
		virtual void LoadTexture(CString aName, bool aTransparent) = 0;
		virtual CTexture GetTexture(CString aName) = 0;
		virtual void SetLinearFiltering(bool aLinearFiltering) = 0;

		virtual void SetBackgroundColour(float aR, float aG, float aB) = 0;
		virtual void BeginFrame() = 0;
		virtual void EndFrame() = 0;

		virtual void SetSpriteViewport(int aXSize, int aYSize) = 0;
		virtual void DrawSprite(CTexture aTexture, float aX, float aY, float aXSize=-1,
				float aYSize=-1, double aXAlign=0, double aYAlign=0) = 0;

		virtual void TriColour(const CVector& aV0, const CVector& aV1, const CVector& aV2,
				const CColour& aColour) = 0;
		virtual void TriColour(const CVector& aV0, const CColour& aC0, const CVector& aV1,
				const CColour& aC1, const CVector& aV2, const CColour& aC2) = 0;
		virtual void TriTexture(const CVector& aV0, float aS0, float aT0,
				const CVector& aV1, float aS1, float aT1,
				const CVector& aV2, float aS2, float aT2, CTexture aTexture) = 0;

		virtual void SetFont(CStringMap<CTexture> aFont) = 0;
		virtual void DrawText(CString aText, int aX, int aY) = 0;

		// audio functions
		virtual void LoadSound(CString aName) = 0;
		virtual void Sound(CString aName) = 0;

		// file functions
		virtual IReader* OpenDataFile(CString aFileName) = 0;
		virtual IReader* ReadUserFile(CString aFileName) = 0;
		virtual IWriter* WriteUserFile(CString aFileName) = 0;
	};
}