#include "Obelisk.h"

#include "CGameController.h"
#include "CMap.h"
#include "CKnifeBot.h"
#include "CBlade.h"
#include "CPowerUp.h"

using namespace Obelisk;



CGameController::CGameController(IObelisk* aObelisk)
		: cXSize(45), cYSize(45)
{
	mObelisk = aObelisk;
	mMap = new CMap(mObelisk, this, cXSize, cYSize);

	// top-down perspective projection matrix
	const double Zn = -2, Zf = -1024;
	double a;
	a = Zf / (Zn - Zf);
	memset(&mProjectionMatrix, 0, 16 * sizeof(float));
	mProjectionMatrix[0 * 4 + 0] = 2 / (double)(1024 / 100);
	mProjectionMatrix[1 * 4 + 1] = -2 * (1 / 0.75) / (double)(1024 / 100);
	mProjectionMatrix[2 * 4 + 2] = a;
	mProjectionMatrix[3 * 4 + 2] = -a * Zn;
	mProjectionMatrix[2 * 4 + 3] = -1;

	glFrontFace(GL_CW);

	mPlayerLastDir = -1;
	mPlayerFrame = 0;
	mPlayerOldInvincible = false;

	mPlayerLeftTexture[0] = mObelisk->GetTexture(SL("Player-Left.bmp"));
	mPlayerLeftTexture[1] = mObelisk->GetTexture(SL("Player-MoveLeft.bmp"));
	mPlayerRightTexture[0] = mObelisk->GetTexture(SL("Player-Right.bmp"));
	mPlayerRightTexture[1] = mObelisk->GetTexture(SL("Player-MoveRight.bmp"));
	mPlayerVerticalTexture[0] = mObelisk->GetTexture(SL("Player-Vertical.bmp"));
	mPlayerVerticalTexture[1] = mObelisk->GetTexture(SL("Player-MoveVertical.bmp"));
	mKnifeBotTexture = mObelisk->GetTexture(SL("KnifeBot.bmp"));
	mBladeTexture = mObelisk->GetTexture(SL("Blade.bmp"));

	mPowerUpTextures[0] = mObelisk->GetTexture(SL("PowerUpExtraLife.bmp"));
	mPowerUpTextures[1] = mObelisk->GetTexture(SL("PowerUpEMP.bmp"));
	mMarkerTextures[0] = mObelisk->GetTexture(SL("MarkerExtraLife.bmp"));
	mMarkerTextures[1] = mObelisk->GetTexture(SL("MarkerEMP.bmp"));

	mTime = mObelisk->CreateTimer();

	mNumEMPs = 4;
	mSpacePressed = false;

	mTimeScore = 0;
	mBladeScore = 0;

	mNextSound = mObelisk->Random(30, 150);
	mBladeCrashSound = 0;
	mEMPSound = 0;
}



CGameController::~CGameController()
{
	delete mMap;
}



bool CGameController::Update()
{
	if (!mMap->IsPlayerLaunched())
	{
		int MoveX = 0;
		if (mObelisk->IsKeyDown(kLeft) != mObelisk->IsKeyDown(kRight))
			MoveX = mObelisk->IsKeyDown(kLeft) ? -1 : 1;
		int MoveY = 0;
		if (mObelisk->IsKeyDown(kUp) != mObelisk->IsKeyDown(kDown))
			MoveY = mObelisk->IsKeyDown(kUp) ? -1 : 1;
		if (MoveX != 0 || MoveY != 0)
		{
			mMap->MovePlayer(MoveX, MoveY);
			if (mMap->GetPlayerDir() != mPlayerLastDir)
			{
				mPlayerLastDir = mMap->GetPlayerDir();
				mPlayerFrame = 1;
				mPlayerFrameCount = 3;
			}
			else
			{
				if (--mPlayerFrameCount == 0)
				{
					mPlayerFrame = 1 - mPlayerFrame;
					mPlayerFrameCount = 3;
				}
			}
		}
		else
		{
			mPlayerLastDir = -1;
			mPlayerFrame = 0;
		}
	}

	if (!mSpacePressed && mObelisk->IsKeyDown(kSpace))
	{
		mSpacePressed = true;
		if (!mMap->IsPlayerLaunched() && mNumEMPs != 0)
		{
			mMap->DetonateEMP(mObelisk, mMap->GetPlayerX(), mMap->GetPlayerY());
			mNumEMPs--;
			mEMPSound = 38;
			mObelisk->Sound(SL("EMP.wav"));
			mNextSound = mObelisk->Random(40, 150);
		}
	}
	else if (mSpacePressed && !mObelisk->IsKeyDown(kSpace))
		mSpacePressed = false;

	mMap->Step(mObelisk);

	CCollection<CKnifeBot>& KnifeBots = mMap->GetKnifeBots();
	foreach (KnifeBots)
	{
		KnifeBots->Step(mObelisk);
	}

	CCollection<CBlade>& Blades = mMap->GetBlades();
	foreach (Blades)
		Blades->Step();

	mMap->CheckCollisions(mObelisk);

	if (mMap->GetPlayerNumLives() == -1)
		return false;

	if (mMap->IsPlayerInvincible() != mPlayerOldInvincible)
	{
		mPlayerOldInvincible = mMap->IsPlayerInvincible();
		if (mMap->IsPlayerInvincible())
		{
			mPlayerInvincibleFrameCount = 2;
			mPlayerInvincibleFrame = 0;
		}
	}
	else if (mMap->IsPlayerInvincible())
	{
		if (--mPlayerInvincibleFrameCount == 0)
		{
			mPlayerInvincibleFrameCount = 2;
			mPlayerInvincibleFrame = 1 - mPlayerInvincibleFrame;
		}
	}

	mTimeScore = mObelisk->Duration(mTime) / 1000;

	if (mEMPSound != 0)
		mEMPSound--;
	else if (mBladeCrashSound != 0)
		mBladeCrashSound--;
	else if (--mNextSound == 0)
	{
		mObelisk->Sound(SL("Robot") + mObelisk->IntToString(mObelisk->Random(0, 3)) + SL(".wav"));
		mNextSound = mObelisk->Random(30, 150);
	}

	return true;
}



void CGameController::Render()
{
	mObelisk->SetLinearFiltering(false);

	glMatrixMode(GL_PROJECTION);
	glLoadIdentity();
	glMultMatrixf(mProjectionMatrix);
	glTranslatef(-mMap->GetPlayerX(), -mMap->GetPlayerY(), -100);

	glMatrixMode(GL_MODELVIEW);
	glLoadIdentity();

	CColour MapColour(.8, .8, .8);
	CVector MapV0(0, 0, 0);
	CVector MapV1(cXSize * 36, 0, 0);
	CVector MapV2(cXSize * 36, cYSize * 36, 0);
	CVector MapV3(0, cYSize * 36, 0);
	mObelisk->TriColour(MapV0, MapV1, MapV2, MapColour);
	mObelisk->TriColour(MapV0, MapV2, MapV3, MapColour);

	// render blades
	CCollection<CBlade>& Blades = mMap->GetBlades();
	foreach (Blades)
	{
		glPushMatrix();
		glTranslatef(Blades->GetX(), Blades->GetY(), 0);
		glRotatef(Blades->GetAngle(), 0, 0, 1);
		CVector V0(-41, -12, 0);
		CVector V1(41, -12, 0);
		CVector V2(41, 12, 0);
		CVector V3(-41, 12, 0);
		mObelisk->TriTexture(V0, 0, 0, V1, 1, 0, V2, 1, 1, mBladeTexture);
		mObelisk->TriTexture(V0, 0, 0, V2, 1, 1, V3, 0, 1, mBladeTexture);
		glPopMatrix();
	}

	CCollection<CPowerUp>& PowerUps = mMap->GetPowerUps();
	foreach (PowerUps)
	{
		glPushMatrix();
		glTranslatef(PowerUps->GetX(), PowerUps->GetY(), 0);
		CVector V0(-9, -9, 0);
		CVector V1(9, -9, 0);
		CVector V2(9, 9, 0);
		CVector V3(-9, 9, 0);
		mObelisk->TriTexture(V0, 0, 0, V1, 1, 0, V2, 1, 1, mPowerUpTextures[PowerUps->GetType()]);
		mObelisk->TriTexture(V0, 0, 0, V2, 1, 1, V3, 0, 1, mPowerUpTextures[PowerUps->GetType()]);
		glPopMatrix();
	}

	// render player
	if (!mMap->IsPlayerInvincible() || mPlayerInvincibleFrame == 0)
	{
		glPushMatrix();
		glTranslatef(mMap->GetPlayerX(), mMap->GetPlayerY(), 0);
		CVector V0(-4, -14, 0);
		CVector V1(4, -14, 0);
		CVector V2(4, 0, 0);
		CVector V3(-4, 0, 0);
		CTexture PlayerTexture;
		if (mMap->GetPlayerDir() == 0)
			PlayerTexture = mPlayerRightTexture[mPlayerFrame];
		else if (mMap->GetPlayerDir() == 2)
			PlayerTexture = mPlayerLeftTexture[mPlayerFrame];
		else
			PlayerTexture = mPlayerVerticalTexture[mPlayerFrame];
		mObelisk->TriTexture(V0, 0, 0, V1, 1, 0, V2, 1, 1, PlayerTexture);
		mObelisk->TriTexture(V0, 0, 0, V2, 1, 1, V3, 0, 1, PlayerTexture);
		glPopMatrix();
	}

	// render knife-bots
	CCollection<CKnifeBot>& KnifeBots = mMap->GetKnifeBots();
	foreach (KnifeBots)
	{
		glPushMatrix();
		glTranslatef(KnifeBots->GetX(), KnifeBots->GetY(), 0);
		glRotatef(KnifeBots->GetAngle(), 0, 0, 1);
		CVector V0(-18, -18, 0);
		CVector V1(18, -18, 0);
		CVector V2(18, 18, 0);
		CVector V3(-18, 18, 0);
		mObelisk->TriTexture(V0, 0, 0, V1, 1, 0, V2, 1, 1, mKnifeBotTexture);
		mObelisk->TriTexture(V0, 0, 0, V2, 1, 1, V3, 0, 1, mKnifeBotTexture);
		glPopMatrix();
	}



	// render power-up markers
	mObelisk->SetLinearFiltering(true);
	foreach (PowerUps)
	{
		if (mMap->DistanceBetween(PowerUps->GetX(), PowerUps->GetY(), mMap->GetPlayerX(),
				mMap->GetPlayerY()) < 200)
			continue;
		double Angle = atan2((double)PowerUps->GetY() - mMap->GetPlayerY(),
				(double)PowerUps->GetX() - mMap->GetPlayerX());
		glPushMatrix();
		glTranslatef(mMap->GetPlayerX() + cos(Angle) * 150, mMap->GetPlayerY() + sin(Angle) * 150, 0);
		glRotatef(Angle * 180 / M_PI, 0, 0, 1);
		CVector V0(-10, -12, 0);
		CVector V1(11, -12, 0);
		CVector V2(11, 12, 0);
		CVector V3(-10, 12, 0);
		mObelisk->TriTexture(V0, 0, 0, V1, 1, 0, V2, 1, 1, mMarkerTextures[PowerUps->GetType()]);
		mObelisk->TriTexture(V0, 0, 0, V2, 1, 1, V3, 0, 1, mMarkerTextures[PowerUps->GetType()]);
		glPopMatrix();
	}
	mObelisk->SetLinearFiltering(false);



	// render UI
	glMatrixMode(GL_PROJECTION);
	glLoadIdentity();
	glOrtho(0, 1024, 768, 0, 0, 1);
	glMatrixMode(GL_MODELVIEW);
	glLoadIdentity();
	for (int i = 0; i < mMap->GetPlayerNumLives(); i++)
	{
		mObelisk->DrawSprite(mObelisk->GetTexture(SL("Player-MoveRight.bmp")),
				1024 - 20 - i * 15, 12, 8, 14, 0, 0);
	}

	for (int i = 0; i < mNumEMPs; i++)
		mObelisk->DrawSprite(mObelisk->GetTexture(SL("EMP.bmp")), 1024 - 20 - i * 15 - 2,
				12 + 22, -1, -1, 0, 0);

	int Score = mTimeScore + mBladeScore;
	mObelisk->DrawText(mObelisk->IntToString(Score), 20, 20);
}



void CGameController::BladeCrash()
{
	if (mEMPSound == 0 && mBladeCrashSound == 0)
	{
		mObelisk->Sound(SL("Blade") + mObelisk->IntToString(mObelisk->Random(0, 3)) + SL(".wav"));
		mBladeCrashSound = 12;
		mNextSound = mObelisk->Random(30, 150);
	}
}