#include <vector>
#include "Obelisk.h"

#include "CMainController.h"
#include "CGameController.h"

using namespace std;
using namespace Obelisk;



IMainController* CreateMainController(IObelisk* aObelisk)
{
	return new CMainController(aObelisk);
}



CMainController::CMainController(IObelisk* aObelisk)
{
	mObelisk = aObelisk;
	mObelisk->SetCaption(SL("Killer Knife-Robots from Knife Planet K"));
	mObelisk->SetSize(1024, 768);
	mObelisk->SetDirectory(SL("KK-RfKPK LD18"));
	mGameController = NULL;
	mHighScore = 0;
}



CMainController::~CMainController()
{
	if (mGameController != NULL)
		delete mGameController;
}



void CMainController::Initialise()
{
	mObelisk->LoadTexture(SL("Title.bmp"), false);
	vector<CString> Textures;
	Textures.push_back(SL("KnifeBot.bmp"));
	Textures.push_back(SL("Blade.bmp"));
	Textures.push_back(SL("Player-Left.bmp"));
	Textures.push_back(SL("Player-MoveLeft.bmp"));
	Textures.push_back(SL("Player-Right.bmp"));
	Textures.push_back(SL("Player-MoveRight.bmp"));
	Textures.push_back(SL("Player-Vertical.bmp"));
	Textures.push_back(SL("Player-MoveVertical.bmp"));
	Textures.push_back(SL("EMP.bmp"));
	Textures.push_back(SL("BestTime.bmp"));
	Textures.push_back(SL("PowerUpExtraLife.bmp"));
	Textures.push_back(SL("PowerUpEMP.bmp"));
	Textures.push_back(SL("MarkerExtraLife.bmp"));
	Textures.push_back(SL("MarkerEMP.bmp"));
	for (int i = 0; i <= 9; i++)
		Textures.push_back(SL("Digit") + mObelisk->IntToString(i) + SL(".bmp"));
	for (int i = 0; i < Textures.size(); i++)
		mObelisk->LoadTexture(Textures[i], true);
	mObelisk->LoadSound(SL("EMP.wav"));
	for (int i = 0; i <= 3; i++)
		mObelisk->LoadSound(SL("Blade") + mObelisk->IntToString(i) + SL(".wav"));
	for (int i = 0; i < 4; i++)
		mObelisk->LoadSound(SL("Robot") + mObelisk->IntToString(i) + SL(".wav"));

	CStringMap<CTexture> Font;
	for (int i = 0; i <= 9; i++)
		Font.Add(mObelisk->IntToString(i), mObelisk->GetTexture(SL("Digit") +
				mObelisk->IntToString(i) + SL(".bmp")));
	mObelisk->SetFont(Font);

	LoadHighScore();

	mObelisk->SetBackgroundColour(.6, .6, .6);
	mObelisk->StartTimer();
}



void CMainController::OnNotify(int aMessage, int aData)
{
}



void CMainController::OnTimer()
{
	if (mGameController == NULL)
	{
		mObelisk->BeginFrame();
		glMatrixMode(GL_PROJECTION);
		glLoadIdentity();
		mObelisk->SetSpriteViewport(1024, 768);
		mObelisk->DrawSprite(mObelisk->GetTexture(SL("Title.bmp")), 512, 384, -1, -1, 0.5, 0.5);
		if (mHighScore != 0)
		{
			mObelisk->DrawSprite(mObelisk->GetTexture(SL("BestTime.bmp")), 342, 700, -1, -1, 0, 0);
			mObelisk->DrawText(mObelisk->IntToString(mHighScore), 342 + 241 + 20, 700);
		}
		mObelisk->EndFrame();
		return;
	}
	if (!mGameController->Update())
	{
		if (mGameController->GetScore() > mHighScore)
		{
			mHighScore = mGameController->GetScore();
			IWriter *Writer = mObelisk->WriteUserFile(SL("HighScore.dat"));
			if (Writer != NULL)
			{
				Writer->Write((TByte*)&mHighScore, sizeof(unsigned int));
				delete Writer;
			}
		}
		delete mGameController;
		mGameController = NULL;
		return;
	}
	mObelisk->BeginFrame();
	mGameController->Render();
	mObelisk->EndFrame();
}



void CMainController::OnDeactivate()
{
}



void CMainController::OnActivate()
{
}



void CMainController::OnKeyDown(Obelisk::EKey aKey)
{
	if (mGameController == NULL)
	{
		if (aKey == kEnter)
			mGameController = new CGameController(mObelisk);
		else if (aKey == kEscape)
			;   // TODO: quit
	}
}



void CMainController::OnKeyUp(Obelisk::EKey aKey)
{
}



void CMainController::ThreadFinished(Obelisk::IThread* aThread)
{
}



void CMainController::LoadHighScore()
{
	IReader *Reader = mObelisk->ReadUserFile(SL("HighScore.dat"));
	if (Reader != NULL)
	{
		unsigned int HighScore = 0;
		if (Reader->Read((TByte*)&HighScore, sizeof(unsigned int)))
			mHighScore = HighScore;
		delete Reader;
	}
}