#include <utility>
#include <vector>

class CGameController;
class CKnifeBot;
class CBlade;
class CPowerUp;

class CMap
{
public:
	CMap(Obelisk::IObelisk* aObelisk, CGameController* aGameController, int aGridXSize, int aGridYSize);
	~CMap();

	int GetXSize() { return mXSize; }
	int GetYSize() { return mYSize; }
	int GetGridXSize() { return mGridXSize; }
	int GetGridYSize() { return mGridYSize; }

	Obelisk::CCollection<CKnifeBot>& GetKnifeBots() { return mKnifeBots; }
	Obelisk::CCollection<CBlade>& GetBlades() { return mBlades; }

	int GetPlayerX() { return mPlayerX; }
	int GetPlayerY() { return mPlayerY; }
	int GetPlayerDir() { return mPlayerDir; }
	int GetPlayerNumLives() { return mPlayerNumLives; }
	bool IsPlayerInvincible() { return mPlayerInvincibleTime != 0; }
	bool IsPlayerLaunched() { return mPlayerLaunchTime != 0; }

	void MovePlayer(int aXDir, int aYDir);

	void CheckCollisions(Obelisk::IObelisk* aObelisk);

	void Step(Obelisk::IObelisk* aObelisk);

	void DetonateEMP(Obelisk::IObelisk* aObelisk, int aX, int aY);

	Obelisk::CCollection<CPowerUp>& GetPowerUps() { return mPowerUps; }

	int DistanceBetween(int aX0, int aY0, int aX1, int aY1);

private:
	CGameController *mGameController;

	Obelisk::CCollection<CKnifeBot> mKnifeBots;
	Obelisk::CCollection<CBlade> mBlades;

	int mGridXSize;
	int mGridYSize;
	int mXSize;
	int mYSize;

	int mPlayerX;
	int mPlayerY;
	int mPlayerDir;
	int mPlayerNumLives;
	int mPlayerInvincibleTime;
	int mPlayerLaunchTime;
	std::pair<float, float> mPlayerVelocity;
	std::pair<float, float> mPlayerRealPos;
	void LaunchPlayer(Obelisk::IObelisk* aObelisk, float aAngle, bool aFast);

	std::vector<std::pair<int, int> > mKnifeBotBoundBox;
	std::pair<int, int> mKnifeBotBoundBoxBox;
	std::vector<std::pair<int, int> > mBladeBoundBox;
	std::pair<int, int> mBladeBoundBoxBox;
	std::vector<std::pair<int, int> > mPlayerBoundBox;
	std::pair<int, int> mPlayerBoundBoxBox;
	std::vector<std::pair<int, int> > mPowerUpBoundBox;
	std::pair<int, int> mPowerUpBoundBoxBox;

	bool Collision(const std::vector<std::pair<int, int> >& aBoundBox0, const std::pair<int, int>& aBoundBoxBox0, int aX0, int aY0, int aAngle0,
			const std::vector<std::pair<int, int> >& aBoundBox1, const std::pair<int, int>& aBoundBoxBox1, int aX1, int aY1, int aAngle1);
	bool Collision1(const std::vector<std::pair<int, int> >& aBoundBox0, int aX0, int aY0, int aAngle0,
			const std::vector<std::pair<int, int> >& aBoundBox1, int aX1, int aY1, int aAngle1);

	std::pair<float, float> TransformPoint(const std::pair<int, int>& p, int aX, int aY, float aAngle);
	std::pair<float, float> GetVector(const std::pair<float, float>& p0, const std::pair<float, float>& p1);

	int mRobotSpawnTime;

	Obelisk::CCollection<CPowerUp> mPowerUps;
	int mPowerUpTime;
};
